@@ -1,7 +1,9 @@
 package App::CPANModulesUtils;
 
-our $DATE = '2020-02-13'; # DATE
-our $VERSION = '0.004'; # VERSION
+our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
+our $DATE = '2021-05-19'; # DATE
+our $DIST = 'App-CPANModulesUtils'; # DIST
+our $VERSION = '0.005'; # VERSION
 
 use 5.010001;
 use strict 'subs', 'vars';
@@ -131,6 +133,135 @@ sub gen_acme_cpanmodules_module_from_acme_cpanlists_list {
     [200];
 }
 
+$SPEC{gen_acme_cpanmodules_module_from_webpage} = {
+    v => 1.1,
+    summary => 'Generate an Acme::CPANModules module file a webpage',
+    description => <<'_',
+
+This routine will download the webpage, extract the links in it, extract the
+CPAN modules mentioned in the links, and use the extracted CPAN modules as
+entries for the Acme::CPANModules module.
+
+_
+    args => {
+        url => {
+            schema => 'str*', # XXX url
+            req => 1,
+            pos => 0,
+        },
+        output_module => {
+            schema => 'perl::modname*',
+            req => 1,
+            pos => 1,
+            description => <<'_',
+
+"Acme::CPANModules::" will be prepended if module name does not begin with it.
+
+_
+        },
+        output_file => {
+            schema => 'filename*',
+            pos => 2,
+            description => <<'_',
+
+will default to "lib/Acme/CPANModules/<YourModuleName>.pm
+
+_
+            #cmdline_aliases => {o=>{}},
+        },
+        overwrite => {
+            schema => 'true*',
+            cmdline_aliases => {O=>{}},
+        },
+    },
+};
+sub gen_acme_cpanmodules_module_from_webpage {
+    require CPAN::Module::FromURL;
+    require Data::Dump;
+    require File::Path;
+    require File::Slurper;
+    require HTTP::Tiny;
+    require HTML::LinkExtor;
+
+    my %args = @_;
+    my $url = $args{url};
+    my $output_module = $args{output_module};
+    $output_module = "Acme::CPANModules::$output_module"
+        unless $output_module =~ /\AAcme::CPANModules::/;
+    #log_trace "output_module=$output_module";
+    my $output_file = $args{output_file} // do {
+        (my $val = "lib/$output_module.pm") =~ s!::!/!g;
+        $val;
+    };
+
+    if (-f $output_file) {
+        $args{overwrite} or return [412, "Output file $output_file already exists, specify another file or --overwrite"];
+    } else {
+        if ($output_file =~ m!/.!) {
+            (my $dir = $output_file) =~ s!(.+)/.+!$1!;
+            File::Path::make_path($dir);
+        }
+    }
+
+    my $list = {
+        summary => $args{summary} // "List of modules mentioned in $url",
+        entries => [],
+    };
+
+    my $res = HTTP::Tiny->new->get($url);
+    return [500, "Couldn't retrieve URL $url: $res->{status} - $res->{reason}"] unless $res->{success};
+
+    my $p = HTML::LinkExtor->new;
+    $p->parse($res->{content});
+
+    my %link_urls;
+    for my $link ($p->links) {
+        my $tag = shift @$link;
+        next unless $tag eq 'a';
+        my %attrs = @$link;
+        #use DD; dd \%attrs;
+        my $link_url = $attrs{href};
+        next if $link_urls{$link_url}++;
+        my $mod = CPAN::Module::FromURL::extract_cpan_module_from_url($link_url);
+        next unless defined $mod;
+        push @{ $list->{entries} }, {module=>$mod};
+    }
+
+    (my $prog = $0) =~ s!.+/!!;
+
+    my $output = join(
+        "",
+
+        "# This file was first automatically generated by ", $prog, " on ", (scalar localtime), " from webpage at ", $url, " by $prog (", __PACKAGE__, " version ", (${__PACKAGE__."::VERSION"} // "dev"), ").\n",
+        "\n",
+
+        "package $output_module;\n",
+        "\n",
+
+        "# AUTHORITY\n",
+        "# DATE\n",
+        "# DIST\n",
+        "# VERSION\n",
+        "\n",
+
+        "our \$LIST = ", Data::Dump::dump($list), ";\n",
+        "\n",
+
+        "1;\n",
+        "# ABSTRACT:\n",
+        "\n",
+
+        "=head1 DESCRIPTION\n\n",
+        "This L<Acme::CPANModules> module was first automatically generated from list of modules mentioned at L<$url>.\n\n",
+
+        "=cut\n\n",
+    );
+
+    log_info "Writing output to $output_file ...";
+    File::Slurper::write_text($output_file, $output);
+    [200];
+}
+
 $SPEC{acme_cpanmodules_for} = {
     v => 1.1,
     summary => 'List Acme::CPANModules distributions that mention specified modules',
@@ -188,7 +319,7 @@ App::CPANModulesUtils - Command-line utilities related to Acme::CPANModules
 
 =head1 VERSION
 
-This document describes version 0.004 of App::CPANModulesUtils (from Perl distribution App-CPANModulesUtils), released on 2020-02-13.
+This document describes version 0.005 of App::CPANModulesUtils (from Perl distribution App-CPANModulesUtils), released on 2021-05-19.
 
 =head1 SYNOPSIS
 
@@ -205,6 +336,8 @@ L<Acme::CPANModules>:
 
 =item * L<gen-acme-cpanmodules-module-from-acme-cpanlists-list>
 
+=item * L<gen-acme-cpanmodules-module-from-webpage>
+
 =back
 
 =head1 FUNCTIONS
@@ -286,6 +419,52 @@ CPANModules.
 =item * B<overwrite> => I<true>
 
 
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (payload) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+
+=head2 gen_acme_cpanmodules_module_from_webpage
+
+Usage:
+
+ gen_acme_cpanmodules_module_from_webpage(%args) -> [status, msg, payload, meta]
+
+Generate an Acme::CPANModules module file a webpage.
+
+This routine will download the webpage, extract the links in it, extract the
+CPAN modules mentioned in the links, and use the extracted CPAN modules as
+entries for the Acme::CPANModules module.
+
+This function is not exported.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<output_file> => I<filename>
+
+will default to "lib/Acme/CPANModules/<YourModuleName>.pm
+
+=item * B<output_module>* => I<perl::modname>
+
+"Acme::CPANModules::" will be prepended if module name does not begin with it.
+
+=item * B<overwrite> => I<true>
+
+=item * B<url>* => I<str>
+
+
 =back
 
 Returns an enveloped result (an array).
@@ -309,7 +488,7 @@ Source repository is at L<https://github.com/perlancar/perl-App-CPANModulesUtils
 
 =head1 BUGS
 
-Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=App-CPANModulesUtils>
+Please report any bugs or feature requests on the bugtracker website L<https://github.com/perlancar/perl-App-CPANModulesUtils/issues>
 
 When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
@@ -325,7 +504,7 @@ perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2020, 2019, 2018 by perlancar@cpan.org.
+This software is copyright (c) 2021, 2020, 2019, 2018 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
